/*=======================================================

	the firmware of USB joystick adapter for Apple II
	version 2
	(c) 2010 Koichi NISHIDA

=======================================================*/

/* supported Apple: Apple II, II plus, and IIe */

/*
hardware information:

use ATTINY2313 AVR.
connect 27MHz (overclock...) crystal to the AVR.
supply 3.3V power.
fuse setting : LOW 11011110

connection:
	PD0: 	connect to USB D+ through 33 ohm register, and pull down with 15k ohm register.
	PD1: 	connect to USB D- through 33 ohm register, and pull down with 15k ohm register .
	PD2:	connect from the analog circuit (see the PNG file).
	PD3-5:	dip switch to show the position of X axis in report data.
			PD5 is msb and PD3 is lsb.
			Y axis position is always the next to the X axis.
	PD6:	dip switch to show which to use, joystick/gamepad(L) or mouse(H)
	PB0-2:	dip switch to show the position of buttons in report data.
			PB2 is msb and PB0 is lsb.
	PB3:	dip switch to show which bit is used for the button data.
			if LOW, use lower 4 bits as button data, otherwise
			use higher 4 bits as button data.
	PB4-5:	connect to the button inputs of APPLE II game port through 100 ohm registers.
			AND, pull down them with 1K ohm registers.
	PB6-7:	input to the analog circuit (see the PNG file).
*/

/*
This is the firmware of USB joystick adapter for APPLE II by Nishida Radio.

Copyright (C) 2009 Koichi NISHIDA
email to Koichi NISHIDA: tulip-house@msf.biglobe.ne.jp

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

	.include "tn2313def.inc"

	;	27MHz	18 clock / bit for 1.5 M BPS

	.equ	MS_COUNT	= (255-13)  ; 27MHz 32u sec
	.equ	TIMER_PARAM	= 3			; prescaler = 64 times

	.equ	TCOUNT		= 0x60
	.equ	TSTATE		= 0x61
	.equ	UCOUNT		= 0x62
	.equ	VALUEX		= 0x63
	.equ	VALUEY		= 0x64
	.equ	VALUEB		= 0x65
	.equ	COUNTX		= 0x66
	.equ	COUNTY		= 0x67
	.equ	DO_FLAG		= 0x68
	.equ	BUFF		= 0x69

	rjmp	VECT_RESET
	rjmp	VECT_INT0
	rjmp	VECT_INT1
	rjmp	VECT_TIMER1_CAPT
	rjmp	VECT_TIMER1_COMPA
	rjmp	VECT_TIMER1_OVF
	rjmp	VECT_TIMER0_OVF
	rjmp	VECT_USART0_RX
	rjmp	VECT_USART0_UDRE
	rjmp	VECT_USART0_TX
	rjmp	VECT_ANALOG_COMP
	rjmp	VECT_PCINT
	rjmp	VECT_TIMER1_COMPB
	rjmp	VECT_TIMER0_COMPA
	rjmp	VECT_TIMER0_COMPB
	rjmp	VECT_USI_START
	rjmp	VECT_USI_OVERFLOW
	rjmp	VECT_EE_READY
	rjmp	VECT_WDT_OVERFLOW

VECT_INT1:
VECT_TIMER1_CAPT:
VECT_TIMER1_COMPA:
VECT_TIMER1_OVF:
VECT_USART0_RX:
VECT_USART0_UDRE:
VECT_USART0_TX:
VECT_ANALOG_COMP:
VECT_PCINT:
VECT_TIMER1_COMPB:
VECT_TIMER0_COMPA:
VECT_TIMER0_COMPB:
VECT_USI_START:
VECT_USI_OVERFLOW:
VECT_EE_READY:
VECT_WDT_OVERFLOW:
	reti

	;
	; convert data on the ROM to the send format
	;
	; inputs:
	;   Z = source data pointer
	; outputs:
	;	*Z = converted data
	;
CONV_SEND:
	inc		ZL
	lpm
	mov		r18,r0
	inc		ZL
	subi	YL,-2
	push	YL
	;		r19					; temp
	;		r20					; source counter
	ldi		r21,1				; first data (phase) = 1
	ldi		r22,0				; six counter
	ldi		r23,0				; target counter
	;		r24					; target data
CNV_SND_LOOP1:
	ldi		r20,8
	lpm
	inc		ZL
CNV_SND_LOOP2:
	mov		r19,r0
	lsl		r0
	andi	r19,0x80
	brne	CNV_SND_NO_CHANGE_PHASE
CNV_SND_CHANGE_POL:				; msb = 0, change phase
	ldi		r22,0
	ldi		r19,1
	eor		r21,r19
	rjmp	CNV_SND_MERGE
CNV_SND_NO_CHANGE_PHASE:		; msb = 1, not change phase
	inc		r22
CNV_SND_MERGE:
	lsl		r24
	or		r24,r21
	inc		r23
	cpi		r23,8
	brne	CNV_SND_NO_STORE
	ldi		r23, 0
	st		Y+,r24
CNV_SND_NO_STORE:
	cpi		r22,6
	breq	CNV_SND_CHANGE_POL
CNV_SND_NO6:																			
	dec		r20
	brne	CNV_SND_LOOP2
	dec		r18
	brne	CNV_SND_LOOP1
	push	r23
	cpi		r23,0
	breq	CNV_SND_END_END
CNV_SND_END_LOOP:
	lsl		r24
	inc		r23
	cpi		r23,8
	brne	CNV_SND_END_LOOP
	st		Y,r24
CNV_SND_END_END:
	pop		r20
	POP		r21
	mov		r19,YL
	sub		r19,r21
	mov		YL,r21
	subi	YL,2
	st		Y+,r19
	st		Y,r20
	ret

	;
	; send data on the RAM to the USB bus
	;
	; inputs:
	;  Y = send data pointer
	;
SEND:
	ld		r19,Y+
	ld		r20,Y+
	ldi		r22,32				; read counter
	lsl		r20
	lsl		r20
	ldi		r23, 0x02			; write data
	out		PORTD, r23
	ldi		r23, 0x3
	out		DDRD, r23
	lsl		r20					; 1
	lsl		r20					; 1
	lsl		r20					; 1
SEND_LOOP:
	nop							; 1
	nop							; 1
	subi	r22,32				; 1
	brne	SEND_WAIT			; 1/2
	ldi		r22,0				; 1
	ld		r21,Y+				; 2
	dec		r19					; 1
	brmi	SEND_END			; 1/2
	brne	SEND_NO_READ		; 1/2
	mov		r22,r20				; 1
SEND_NO_READ:
	ldi		r23,1				; 1
	sbrc	r21,7				; 1
	ldi		r23,2				; 1
	out		PORTD,r23			; 1
	lsl		r21					; 1
	rjmp	SEND_LOOP			; 2
SEND_WAIT:
	nop							; 1
	nop							; 1
	nop							; 1
	nop							; 1
	rjmp	SEND_NO_READ		; 2
SEND_EOP:
	ldi		r18, 0x02			; write data
	out		PORTD, r18
	ldi		r18, 0x3			; 1
	out		DDRD, r18			; 1
	nop							; 1
	ldi		r19,4				; 1
SEND_EOP_LOOP:
	dec		r19					; 1
	brne	SEND_EOP_LOOP		; 2/1
SEND_END:
	nop							; 1
	nop							; 1
	nop							; 1
	ldi		r19,0				; 1
	out		PORTD,r19			; 1
	ldi		r19,11				; 1
SEND_END_LOOP1:
	dec		r19					; 1
	brne	SEND_END_LOOP1		; 2/1
	nop							; 1
	ldi		r19,2				; 1
	out		PORTD,r19			; 1
	ldi		r19,11				; 1
SEND_END_LOOP2:
	dec		r19					; 1
	brne	SEND_END_LOOP2		; 2/1
	nop							; 1
	ldi		r19, 0x0			; 1
	out		DDRD, r19
	out		PORTD, r19
	ret

;
;	receive data from USB
;	  input = Z : destination pointer
;	  output = *ZL : received data
;
RECEIVE:
	push	ZL
	inc		ZL
	ldi		r19, 0
	ldi		r20, 0xff
	ldi		r21, 0xff
RECEIVE_WAIT:
	sbic	PIND, 1
	rjmp	RECEIVE_WAIT
RECEIVE_LOOP:
	in		r18, PIND			; 1
	andi	r18, 0x03 			; 1
	breq	RECEIVE_END_LOOP	; 1
	cp		r18,r20				; 1
	mov		r20,r18				; 1
	breq	RECEIVE_EQUAL		; 1/2
RECEIVE_NOT_EQUAL:
	andi	r21,0x3				; 1
	cp		r18,r21				; 1
	brne	RECEIVE_LONGER		; 1/2
RECEIVE_SHORTER:
	ror		r18					; 1
	ror		r22					; 1
	subi	r19,32				; 1
	in		r21,PIND			; 1
	brne	RECEIVE_NO_STORE	; 1/2
	st		Z+,r22				; 2
	rjmp	RECEIVE_LOOP		; 2
RECEIVE_NO_STORE_L:
	nop							; 1
RECEIVE_NO_STORE:
	rjmp	RECEIVE_LOOP		; 2
RECEIVE_EQUAL:
	nop							; 1
	nop							; 1
RECEIVE_LONGER:
	ror		r18					; 1
	ror		r22					; 1
	subi	r19,32				; 1
	in		r21,PIND			; 1
	brne	RECEIVE_NO_STORE_L	; 1/2
	st		Z+,r22				; 2
	rjmp	RECEIVE_LOOP		; 2
RECEIVE_END_LOOP:
	sbis	PIND, 1
	rjmp	RECEIVE_END_LOOP
	mov		r18,ZL
	subi	r18,LOW(BUFF)
	dec		r18
	pop		ZL
	st		Z,r18
	ret

;
; convert read data to actual data
;   input = Y : source pointer
;	        Z : destination pointer
;   output = *Z : converted data
;
CONV_RECEIVE:
	push	r18
	ld		r18,Y+
	ldi		r19, 1				; source counter
	ldi		r20, 8				; destination counter
	ldi		r23, 0xff			; previous value
	ldi		r24, 0				; NZRI count
	ldi		r25, 0				; result value
								; r21 : source value
								; r22 : temprorary
CONV_RCV_LOOP:
	dec		r19
	brne	CONV_RCV_NO_LOAD
	dec 	r18
	breq	CONV_RCV_END
	ldi		r19,8
	ld		r21, Y+
CONV_RCV_NO_LOAD:
	cpi		r24,6
	brne	CONV_RCV_UNDER6
	mov		r22,r21
	ror		r21
	andi	r22,1
	mov		r23,r22
	ldi		r24,0
	rjmp	CONV_RCV_LOOP
CONV_RCV_UNDER6:
	mov		r22,r21
	ror		r21
	andi	r22,1
	cp		r23,r22
	brne	CONV_RCV_NE
CONV_RCV_EQ:
	inc		r24
	sec
	rjmp	CONV_RCV_MERGE
CONV_RCV_NE:
	ldi		r24,0
	clc
CONV_RCV_MERGE:
	mov		r23,r22
	ror		r25
	dec		r20
	brne	CONV_RCV_LOOP
	ldi		r20,8
	st		Z+, r25
	rjmp	CONV_RCV_LOOP
CONV_RCV_END:
	pop		r18
	ret

; ========== get report descriptor ==========
GET_REPORT:
	; none ...
	ldi		ZL,LOW(INIT_RETURN)
	ldi		ZH,HIGH(INIT_RETURN)
	ijmp

; ========== set address ==========
SET_ADDRESS:
	; setup stage

	ldi		ZH, HIGH(PKT_SETUP_0*2)
	ldi		ZL, LOW(PKT_SETUP_0*2)
	ldi		YL,LOW(BUFF)
	rcall	CONV_SEND

	ldi		ZH, HIGH(PKT_SET_ADDRESS*2)
	ldi		ZL, LOW(PKT_SET_ADDRESS*2)
	ldi		YL,LOW(BUFF)+6
	rcall	CONV_SEND

	ldi		YL,LOW(BUFF)
	rcall	SEND

	ldi		YL,LOW(BUFF)+6	
	rcall	SEND

	ldi		ZL, LOW(BUFF)		;	should be ack
	rcall	RECEIVE

	; no data stage

	; status stage
SET_ADDRESS_NAK:
	ldi		ZH, HIGH(PKT_IN_0_0*2)
	ldi		ZL, LOW(PKT_IN_0_0*2)
	ldi		YL,LOW(BUFF)
	rcall	CONV_SEND

	ldi		ZH, HIGH(PKT_ACK*2)
	ldi		ZL, LOW(PKT_ACK*2)
	ldi		YL,LOW(BUFF+14)	
	rcall	CONV_SEND

	ldi		YL,LOW(BUFF)
	rcall	SEND

	ldi		ZL, LOW(BUFF)
	rcall	RECEIVE

	lds		r18,BUFF+2
	andi	r18,0x3
	breq	SET_ADDRESS_NAK
	
	; ack
	ldi		YL,LOW(BUFF+14)	
	rcall	SEND

	ldi		ZL,LOW(INIT_RETURN)
	ldi		ZH,HIGH(INIT_RETURN)
	ijmp

; ========== set configuration ==========
SET_CONFIGURATION:
	; setup stage
	ldi		ZH, HIGH(PKT_SETUP_1*2)
	ldi		ZL, LOW(PKT_SETUP_1*2)
	ldi		YL,LOW(BUFF)	
	rcall	CONV_SEND

	ldi		ZH, HIGH(PKT_SET_CONFIGURATION*2)
	ldi		ZL, LOW(PKT_SET_CONFIGURATION*2)
	ldi		YL,LOW(BUFF)+6
	rcall	CONV_SEND

	ldi		YL,LOW(BUFF)	
	rcall	SEND

	ldi		YL,LOW(BUFF)+6
	rcall	SEND

	ldi		ZL, LOW(BUFF)
	rcall	RECEIVE

	; no data stage

	; status stage
SET_CONF_NAK:
	ldi		ZH, HIGH(PKT_IN_1_0*2)
	ldi		ZL, LOW(PKT_IN_1_0*2)	
	ldi		YL,LOW(BUFF)	
	rcall	CONV_SEND

	ldi		ZH, HIGH(PKT_ACK*2)
	ldi		ZL, LOW(PKT_ACK*2)
	ldi		YL,LOW(BUFF+14)
	rcall	CONV_SEND

	ldi		YL,LOW(BUFF)		; pkt in
	rcall	SEND

	ldi		ZL, LOW(BUFF)
	rcall	RECEIVE

	lds		r18,BUFF+2
	andi	r18,0x3
	breq	SET_CONF_NAK

	ldi		YL,LOW(BUFF+14)		; ack
	rcall	SEND

	ldi		ZL,LOW(INIT_RETURN)
	ldi		ZH,HIGH(INIT_RETURN)
	ijmp

; ========== interrupt transfer ==========
INTR_TRANSFER:
	ldi		ZH, HIGH(PKT_IN_1_1*2)
	ldi		ZL, LOW(PKT_IN_1_1*2)
	ldi		YL,LOW(BUFF)	
	rcall	CONV_SEND

	ldi		ZH, HIGH(PKT_ACK*2)
	ldi		ZL, LOW(PKT_ACK*2)
	ldi		YL,LOW(BUFF+17)
	rcall	CONV_SEND

	cli

	ldi		YL,LOW(BUFF)
	rcall	SEND

	ldi		ZL, LOW(BUFF+4)		; destination
	rcall	RECEIVE

	; ack
	ldi		YL,LOW(BUFF+17)	
	rcall	SEND

	sei

	; return to main if nak, stall nor not yet
	lds		r18,BUFF+6
	andi	r18,3

	brne	NO_EXIT
	ret
NO_EXIT:
	ldi		YL, LOW(BUFF+4)		; source
	ldi		ZL, LOW(BUFF+4)		; destination
	rcall	CONV_RECEIVE

	lds		r18,BUFF+5
	cpi		r18,0b11000011
	breq	STORE_DATA
	cpi		r18,0b01001011
	breq	STORE_DATA
	ret
STORE_DATA:
	ldi		YL, LOW(BUFF+6)
	in		r19,PIND
	lsr		r19
	lsr		r19
	lsr		r19
	andi	r19,7
	add		YL,r19
	in		r20,PIND
	andi	r20,0b01000000
	breq	JOYSTICK
MOUSE:
	lds		r18,VALUEX
	ld		r19, Y
	asr		r19
	asr		r19
	cpi		r19,0x80
	brcc	MOUSE_MINUS_X
MOUSE_PLUS_X:
	add		r18,r19
	brcc	MOUSE_X_MERGE
	ldi		r18,0xff
	rjmp	MOUSE_X_MERGE
MOUSE_MINUS_X:
	add		r18,r19
	brcs	MOUSE_X_MERGE
	ldi		r18,0
MOUSE_X_MERGE:
	sts		VALUEX,r18

	inc		YL
	lds		r18,VALUEY
	ld		r19, Y
	asr		r19
	asr		r19
	cpi		r19,0x80
	brcc	MOUSE_MINUS_Y
MOUSE_PLUS_Y:
	add		r18,r19
	brcc	MOUSE_Y_MERGE
	ldi		r18,0xff
	rjmp	MOUSE_Y_MERGE
MOUSE_MINUS_Y:
	add		r18,r19
	brcs	MOUSE_Y_MERGE
	ldi		r18,0
MOUSE_Y_MERGE:
	sts		VALUEY,r18
	rjmp	MOUSE_JOY_MERGE

JOYSTICK:
	ld		r18, Y
	sts		VALUEX,r18
	inc		YL
	ld		r18,Y
	sts		VALUEY,r18
MOUSE_JOY_MERGE:
	ldi		YL, LOW(BUFF+6)
	in		r19,PINB
	andi	r19,7
	add		YL,r19
	ld		r18,Y
	sts		VALUEB,r18
INTR_EXIT:
	ret

; ========== timer interrupt ==========
VECT_TIMER0_OVF:	; every 32u sec here
	push	r18
	in		r18,SREG
	push	r18
	push	r19
	push	r20
	push	r21
	push	r22

	ldi		r18, MS_COUNT
	out		TCNT0, r18

	; DO_FLAG gets on every 1ms
	lds		r18,UCOUNT
	inc		r18
	cpi		r18,32
	brne	UCOUNT_NO_RESET
	ldi		r18,0
	ldi		r19,1
	sts		DO_FLAG, r19
UCOUNT_NO_RESET:
	sts		UCOUNT,r18

	lds		r20,COUNTX
	cpi		r20,253
	brcs	COUNT_CARRY_X
	ldi		r20,252
COUNT_CARRY_X:
	subi	r20,-3
	sts		COUNTX,r20

	lds		r22,COUNTY
	cpi		r22,253
	brcs	COUNT_CARRY_Y
	ldi		r22,252
COUNT_CARRY_Y:
	subi	r22,-3
	sts		COUNTY,r22

	lds		r19,VALUEY
	cp		r22,r19
	brcs	COUNT_NOT_RESET_Y0
	ldi		r18,0b10110000
	out		PORTB,r18

	nop
	nop
	nop
	nop
	nop
	nop

	andi	r18,0b00111111
	out		PORTB,r18
	ldi		r18,0
	sts		COUNTY,r18
COUNT_NOT_RESET_Y0:

	lds		r19,VALUEX
	cp		r20,r19
	brcs	COUNT_NOT_RESET_X0
	ldi		r18,0b01110000
	out		PORTB,r18

	nop
	nop
	nop
	nop
	nop
	nop

	andi	r18,0b00111111
	out		PORTB,r18
	ldi		r18,0
	sts		COUNTX,r18
COUNT_NOT_RESET_X0:

	pop		r22
	pop		r21
	pop		r20
	pop		r19
	pop		r18
	out		SREG,r18
	pop		r18
	reti

; ========== MAIN ENTRANCE ==========
VECT_RESET:
	; wait for a while
	ldi		r18,255
DW1:
	ldi 	r19,255
DW2:
	nop
	dec		r19
	brne	DW2
	dec		r18
	brne	DW1

	; ports & stack pointer setting
	ldi		r18, LOW(RAMEND)
	out		SPL, r18

	ldi		r18, 0x0
	out		DDRD, r18
	ldi		r18, 0x00
	out		PORTD, r18

	ldi		r18,0b00110000
	out		PORTB, r18
	ldi		r18, 0xc0
	out		DDRB, r18

	; init variables
	ldi		r18,0
	sts		TCOUNT,r18
	sts		TSTATE,r18
	sts		UCOUNT,r18
	sts		COUNTX,r18
	sts		COUNTY,r18
	sts		DO_FLAG,r18
	sts		VALUEB,r18
	ldi		r18,128
	sts		VALUEX,r18
	sts		VALUEY,r18

	; set timer as 32u sec
	ldi		r18,0
	out		OCR0A, r18
	ldi		r18,0b0
	out		TCCR0A, r18
	ldi		r18, TIMER_PARAM
	out		TCCR0B, r18
	ldi		r18,2
	out		TIMSK, r18
	ldi		r18, MS_COUNT
	out		TCNT0, r18

	; INT0 falling edge interrupt
	ldi		r18, 2
	out		MCUCR,r18

	; enable INT0
	ldi		r18,0x40
	out		GIMSK,r18

	sei

MAIN_LOOP: ;========= MAIN LOOP ===========
	lds		r18,DO_FLAG
	cpi		r18,1
	brcs	MAIN_LOOP

DO_USB:
	; every 1ms here
	lds		r18,TSTATE
	cpi		r18,205
	brcs	NOT_SEND_EOP
	rcall	SEND_EOP
	lds		r18,TSTATE
NOT_SEND_EOP:
	cpi		r18,217
	brne	INITIALIZE
	lds		r18,TCOUNT
	inc		r18
	sts		TCOUNT,r18
	cpi		r18,10
	breq	TIM_CONT
	rjmp	USB_EXIT
TIM_CONT:
	; every 10ms here
	ldi		r18,0
	sts		TCOUNT,r18
	rcall	INTR_TRANSFER		; receive DATA from USB
	rjmp	USB_EXIT
INITIALIZE:
	cli
	cpi		r18,200
	brne	INIT1
	ldi		r18, 0x3
	out		DDRD, r18
	ldi		r18, 0x00
	out		PORTD, r18
	rjmp	INIT_RETURN
INIT1:cpi	r18,204
	brne	INIT2
	ldi		r18, 0x0
	out		DDRD, r18
	ldi		r18, 0x00
	out		PORTD, r18
	rjmp	INIT_RETURN
INIT2:cpi	r18,214
	brne	INIT3
	ldi		ZL,LOW(GET_REPORT)
	ldi		ZH,HIGH(GET_REPORT)
	ijmp
INIT3:cpi		r18,215
	brne	INIT4
	ldi		ZL,LOW(SET_ADDRESS)
	ldi		ZH,HIGH(SET_ADDRESS)
	ijmp
INIT4:cpi		r18,216
	brne	INIT_RETURN
	ldi		ZL,LOW(SET_CONFIGURATION)
	ldi		ZH,HIGH(SET_CONFIGURATION)
	ijmp
INIT_RETURN:
	; state transition
	lds		r18,TSTATE
	inc		r18
	sts		TSTATE,r18
	sei
USB_EXIT:

DO_BUTTON:
	ldi		r22,0b00001001
	ldi		r23,0b00000110
	in		r18,PINB
	andi	r18,0b00001000
	breq	NOT_CHANGE_BUTTON
	ldi		r22,0b10010000
	ldi		r23,0b01100000
NOT_CHANGE_BUTTON:
	lds		r18,VALUEB
	ldi		r21,0b00000000

	push	r18

	and		r18,r22
	breq	BT0_ZERO
	sbi		DDRB,4
	rjmp	BT0_ZERO_1
BT0_ZERO:
	cbi		DDRB,4
BT0_ZERO_1:

	pop		r18

	and		r18,r23
	breq	BT1_ZERO
	sbi		DDRB,5
	rjmp	BT1_ZERO_1
BT1_ZERO:
	cbi		DDRB,5
BT1_ZERO_1:

	lds		r18,0
	sts		DO_FLAG,r18
	rjmp	MAIN_LOOP

; ========== INT0 interrupt ==========
VECT_INT0:
	push	r18
	ldi		r18,0
	sts		COUNTX,r18
	sts		COUNTY,r18
	pop		r18
	reti

	.org 0x400-50

PKT_SOF:
	.db		0,4
	.db		0b00000001,0b10100101	;	SYNC, SOF
	.db		0b00000000,0b00001000	;
PKT_ACK:
	.db		0,2
	.db		0b00000001,0b01001011	;	SYNC, ACK
PKT_SETUP_0:
	.db		0,4
	.db		0b00000001,0b10110100	;	SYNC, SETUP
	.db		0b00000000,0b00001000	;	ADDR = 0, ENDP = 0, CRC5
PKT_GET_REPORT:
	.db		0,12
	.db		0b00000001,0b11000011	;	SYNC, DATA0
	.db		0b00000001,0b01100000	;	In | Standard Request, GET_DESCRIPTOR(6)
	.db		0b00000000,0b01000100	;	wValue = 0x22
	.db		0b00000000,0b00000000	;	wIndex = 0
	.db		0b00010000,0b00000000	;	wLength = 8
	.db		0b01110100,0b11001010   ;	CRC16
PKT_SET_ADDRESS:
	.db		0,12
	.db		0b00000001,0b11000011	;	SYNC, DATA0
	.db		0b00000000,0b10100000	;	OUT | Standard Request, SET_ADDRESS(5)
	.db		0b10000000,0b00000000	;	wValue = 1
	.db		0b00000000,0b00000000
	.db		0b00000000,0b00000000
	.db		0b11010111,0b10100100	;	CRC16	
PKT_SETUP_1:
	.db		0,4
	.db		0b00000001,0b10110100	;	SYNC, SETUP
	.db		0b10000000,0b00010111	;	ADDR = 1, ENDP = 0, CRC
PKT_SET_CONFIGURATION:
	.db		0,12
	.db		0b00000001,0b11000011	;	SYNC, DATA0
	.db		0b00000000,0b10010000	;	Out | Standard Request, SET_CONFIGURATION(9)
	.db		0b10000000,0b00000000	;	wValue = 1
	.db		0b00000000,0b00000000
	.db		0b00000000,0b00000000
	.db		0b11100100,0b10100100	;	CRC16
PKT_DUMMY_OUT:
	.db		0,4
	.db		0b00000001,0b11010010	;	SYNC, DATA1
	.db		0b00000000,0b00000000	;	CRC16
PKT_OUT_0_0:
	.db		0,4
	.db		0b00000001,0b10000111	;	SYNC, PID_OUT
	.db		0b00000000,0b00001000	;	ADDR = 0, ENDP = 0, CRC
PKT_IN_0_0:
	.db		0,4
	.db		0b00000001,0b10010110	;	SYNC, PID_IN
	.db		0b00000000,0b00001000	;	ADDR = 0, ENDP = 0, CRC
PKT_IN_0_1:
	.db		0,4
	.db		0b00000001,0b10010110	;	SYNC, PID_IN
	.db		0b00000000,0b00000101	;	ADDR = 0, ENDP = 1, CRC
PKT_IN_1_0:
	.db		0,4
	.db		0b00000001,0b10010110	;	SYNC, PID_IN
	.db		0b10000000,0b00010111	;	ADDR = 1, ENDP = 0, CRC
PKT_IN_1_1:
	.db		0,4
	.db		0b00000001,0b10010110	;	SYNC, PID_IN
	.db		0b10000001,0b00011010	;	ADDR = 1, ENDP = 1, CRC
